var module = angular.module("Mascarille", ['ui.router',
    'picardy.fontawesome',
    'ngMaterial',
]);


// add constant
module.constant("API_INIT", "http://localhost:5000/init");

module.controller('AppController', function($scope, $mdSidenav, $mdToast, $location, $state, $mdDialog, $http, API_INIT) {
    // commun
    $scope.clearObj = function(obj) {
        for (prop in obj) {
            if (obj.hasOwnProperty(prop)) {
                delete obj[prop];
            }
        };
    };
    // manage visual
    $scope.openLeftMenu = function() {
        $mdSidenav('left-menu').toggle();
    };
    $scope.toggleFiltered = function() {
        $scope.hasFiltered = !$scope.hasFiltered;
    };
    $scope.onlySelectedOrNot = function(item) {
        if (!$scope.hasFiltered) {
            return true;
        } else {
            if (item.selected) {
                return true;
            } else {
                false;
            };
        };
    };
    $scope.toggleSelected = function(item) {
        item.selected = !item.selected;
        $scope.hasSelected = false;
        for (obj in $scope.resultsearch) {
            if ($scope.resultsearch[obj].selected && !$scope.hasSelected) {
                $scope.hasSelected = true;
            };
        };
        if (!$scope.hasSelected && $scope.hasFiltered) {
            $scope.toggleFiltered();
        };
    };
    $scope.viewSelected = function(item) {
        var parentEl = angular.element(document.body);
        $mdDialog.show({
            parent: parentEl,
            escapeToClose: true,
            templateUrl: $scope.getEnv('viewDetail'),
            controller: DialogController,
            locals: {
                item: item
            },
            scope: $scope,
            preserveScope: true
        });

        function DialogController($scope, $mdDialog, item) {
            $scope.initItem = function() {
                $scope.item.nextitem = $scope.nextItem($scope.item);
                $scope.item.previtem = $scope.prevItem($scope.item);
            };
            $scope.closeDialog = function() {
                $mdDialog.hide();
            };
            $scope.toggle = function() {
                $scope.toggleSelected($scope.item);
            };
            $scope.nextItem = function(item) {
                var find = false;
                for (obj in $scope.resultsearch) {
                    if (find && $scope.onlySelectedOrNot($scope.resultsearch[obj])) {
                        return $scope.resultsearch[obj];
                    };
                    if (!find && $scope.resultsearch[obj].id == item.id) {
                        find = true;
                    };
                }
                return undefined;
            };
            $scope.prevItem = function(item) {
                var prev = undefined;
                for (obj in $scope.resultsearch) {
                    if ($scope.resultsearch[obj].id == item.id && prev) {
                        return $scope.resultsearch[prev];
                    };
                    if ($scope.onlySelectedOrNot($scope.resultsearch[obj])) {
                        prev = obj;                        
                    }
                }
                return undefined;
            };
            $scope.backDialog = function() {
                if ($scope.item.previtem) {
                    $scope.item = $scope.item.previtem;
                    $scope.initItem();
                };
            };
            $scope.forDialog = function() {
                if ($scope.item.nextitem) {
                    $scope.item = $scope.item.nextitem;
                    $scope.initItem();
                };
            };
            $scope.item = item;
            $scope.initItem();
        }
    };
    $scope.toggleAdvanceSearch = function() {
        $scope.showAdvanceSearch = !$scope.showAdvanceSearch;
    };
    $scope.openAdvanceSearch = function() {
        $scope.showAdvanceSearch = true;
    };
    $scope.closeAdvanceSearch = function() {
        $scope.showAdvanceSearch = false;
    };
    $scope.showNavIcon = function() {
        return !$mdSidenav('left-menu').isLockedOpen();
    };
    $scope.getEnv = function(prop) {
        if ($state.current['name']) {
            return $scope.menus[$state.current['name']][prop];
        };
        /* ------------------------------------------------------------------ */
        /* /!\ comment faire quand $state.current['name'] ne ramene rien */
        /* /!\ mettre author en dure cela n est pas cool */
        return $scope.menus['author'][prop];
        /* ------------------------------------------------------------------ */
    };
    $scope.getCurrent = function() {
        return $scope.getEnv('class');
    };
    $scope.navigateTo = function(to) {
        $state.go(to);
        if ($scope.showNavIcon() && $mdSidenav('left-menu').isOpen()) {
            $scope.openLeftMenu();
        };
    };
    $scope.showToast = function(msg, level) {
        $mdToast.show({
            templateUrl: 'app/templates/toast-' + level + '.html',
            controller: function($scope) {
                $scope.msg = msg;
            },
            hideDelay: 6000,
            position: 'bottom right'
        });
    };

    //manage webservice
    $scope.getinit = function() {
        $http.get(API_INIT).
        success(function(data, status, headers, config) {
            $scope.init = data;
            $scope.showToast("init ok", "success");
        }).
        error(function(data, status, headers, config) {
            $scope.showToast("init error", "error");
        });
    };

    $scope.searchGo = function() {
        if ($scope.init) {
            $scope.clearObj($scope.resultsearch);
            var api = $scope.init['api'][$scope.getEnv('api')];
            $http.post(api, {
                'search': $scope.search
            }).
            success(function(data, status, headers, config) {
                $scope.resultsearch = data[0]['result'];
                $scope.showToast(data[0]['msg'], "info");
            }).
            error(function(data, status, headers, config) {
                $scope.showToast("searchGo error " + api, "error");
            });
        } else {
            $scope.showToast("searchGo init error", "error");
        };
    };

    // param app
    $scope.menus = {
        "author": {
            "icon": "search",
            "title": "Auteur",
            "to": "author",
            "class": "author",
            "api": 'author',
            "viewDetail": './app/templates/author.detail.html'

        },
        "play": {
            "icon": "search",
            "title": "Pièces de Théâtre",
            "to": "play",
            "class": "play",
            "api": "play",
            "viewDetail": './app/templates/play.detail.html'

        },
        "book": {
            "icon": "search",
            "title": "Ouvrage sur le Théâtre",
            "to": "book",
            "class": "book",
            "api": "book",
            "viewDetail": './app/templates/book.detail.html'

        },
        "publisher": {
            "icon": "search",
            "title": "Editeur",
            "to": "publisher",
            "class": "publisher",
            "api": "publisher",
            "viewDetail": './app/templates/publisher.detail.html'

        }
    };
    $scope.search = {};
    $scope.resultsearch = {};
    $scope.hasSelected = false;
    $scope.hasFiltered = false;

    // action init
    $scope.closeAdvanceSearch();
    $scope.getinit();

});


module.controller('RouteController', function($scope, $state) {
    $scope.closeAdvanceSearch();
    $scope.clearObj($scope.resultsearch);
    $scope.clearObj($scope.search);
    $scope.hasSelected = false;
    $scope.hasFiltered = false;
});



//add route
module.config(function($stateProvider, $urlRouterProvider) {
    // Now set up the states
    $stateProvider.state('author', {
        url: "/author",
        views: {
            'content': {
                templateUrl: "./app/templates/author.html",
                controller: 'RouteController'
            },
            'bar': {
                templateUrl: "./app/templates/author.bar.html",
                controller: 'RouteController'
            }
        }
    });
    $stateProvider.state('play', {
        url: "/play",
        views: {
            'content': {
                templateUrl: "./app/templates/play.html",
                controller: 'RouteController'
            },
            'bar': {
                templateUrl: "./app/templates/play.bar.html",
                controller: 'RouteController'
            }
        }
    });
    $stateProvider.state('book', {
        url: "/book",
        views: {
            'content': {
                templateUrl: "./app/templates/book.html",
                controller: 'RouteController'
            },
            'bar': {
                templateUrl: "./app/templates/book.bar.html",
                controller: 'RouteController'
            }
        }
    });
    $stateProvider.state('publisher', {
        url: "/publisher",
        views: {
            'content': {
                templateUrl: "./app/templates/publisher.html",
                controller: 'RouteController'
            },
            'bar': {
                templateUrl: "./app/templates/publisher.bar.html",
                controller: 'RouteController'
            }
        }
    });
    $urlRouterProvider.otherwise("/author");
});
